#ifndef _CLM_H_
#define _CLM_H_

#include <stdbool.h>

typedef struct {
	float *values;
	unsigned int rows;
	unsigned int cols;
} clm_Matrix;

typedef struct {
	float *values;
	unsigned int length;
} clm_Vector;

typedef struct {
	clm_Matrix weights;
	clm_Matrix bias;
} clm_Linear;

typedef struct {
	clm_Linear *layers;
	unsigned int numLayers;
	float learnRate;
} clm_NN;

extern const clm_Matrix INVALID_MATRIX;
extern const clm_Vector INVALID_VECTOR;

clm_Matrix clm_createMatrixRandom(unsigned int rows, unsigned int cols);
clm_Matrix clm_matrixAddScalar(clm_Matrix mat, float scalar);
clm_Matrix clm_matrixAddMatrix(clm_Matrix mat, clm_Matrix other);
clm_Matrix clm_matrixSubtractMatrix(clm_Matrix mat, clm_Matrix other);
clm_Matrix clm_matrixTranspose(clm_Matrix mat);
clm_Matrix clm_matrixMultiplyMatrix(clm_Matrix a, clm_Matrix b);
clm_Matrix clm_matrixMultiplyMatrixElements(clm_Matrix mat, clm_Matrix other);
clm_Matrix clm_matrixMultiplyScalar(clm_Matrix mat, float scalar);
clm_Matrix clm_matrixSigmoid(clm_Matrix mat);
clm_Matrix clm_matrixDSigmoid(clm_Matrix mat);

clm_Matrix clm_matrixFromArray(float *array, unsigned int length);
clm_Matrix clm_matrixCopy(clm_Matrix matrix);

bool clm_matrixIsInvalid(clm_Matrix mat);

clm_Vector clm_vectorCreate(unsigned int length);

bool clm_vectorIsInvalid(clm_Vector vec);

clm_Linear clm_linearCreateRandom(unsigned int inputs, unsigned int outputs);

void clm_matrixPrint(clm_Matrix mat);
void clm_matrixPrintShape(clm_Matrix mat);
void clm_freeMatrix(clm_Matrix mat);
void clm_freeVector(clm_Vector vector);
void clm_freeLinear(clm_Linear linear);

#endif
